package com.ruoyi.project.business.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import com.ruoyi.common.enums.DelFlag;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.business.domain.TbArea;
import com.ruoyi.project.business.domain.vo.country.AreaWithCountriesVo;
import com.ruoyi.project.business.domain.vo.selectParam.AreaParam;
import com.ruoyi.project.business.mapper.TbAgentMapper;
import com.ruoyi.project.business.mapper.TbCountryAreaMapper;
import com.ruoyi.project.business.mapper.TcAccountMapper;
import com.ruoyi.project.common.service.ICommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import com.ruoyi.project.business.mapper.TbAreaMapper;
import com.ruoyi.project.business.service.ITbAreaService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import static com.ruoyi.common.constant.CacheConstants.AREA_KEY;

/**
 * 区域Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-04-15
 */
@Service
public class TbAreaServiceImpl implements ITbAreaService
{
    @Autowired
    private TbAreaMapper tbAreaMapper;

    @Resource
    private ICommonService commonService;

    @Resource
    private TbCountryAreaMapper tbCountryAreaMapper;

    @Resource
    private TcAccountMapper tcAccountMapper;

    @Resource
    private TbAgentMapper tbAgentMapper;

    /**
     * 查询区域
     * 
     * @param areaId 区域主键
     * @return 区域
     */
    @Override
    public TbArea selectTbAreaByAreaId(Long areaId)
    {
        // 校验数据是否已被删除
        TbArea tbArea = tbAreaMapper.selectTbAreaByAreaId(areaId);
        if (DelFlag.YES.getCode().equals(tbArea.getDelFlag())) {
            throw new ServiceException(MessageUtils.message("business.data.deleted"));
        }
        return tbArea;
    }

    /**
     * 查询区域列表
     * 
     * @param tbArea 区域
     * @return 区域
     */
    @Override
    public List<TbArea> selectTbAreaList(AreaParam tbArea)
    {
        if (StringUtils.isNotEmpty(tbArea.getAreaName())) {
            tbArea.setAreaName(tbArea.getAreaName().replaceAll("%", "/%").replaceAll("_", "/_"));
        }
        if (StringUtils.isNotEmpty(tbArea.getAreaNameUS())) {
            tbArea.setAreaNameUS(tbArea.getAreaNameUS().replaceAll("%", "/%").replaceAll("_", "/_"));
        }
        if (StringUtils.isNotEmpty(tbArea.getAreaNameRU())) {
            tbArea.setAreaNameRU(tbArea.getAreaNameRU().replaceAll("%", "/%").replaceAll("_", "/_"));
        }
        if (StringUtils.isNotEmpty(tbArea.getAreaNameES())) {
            tbArea.setAreaNameES(tbArea.getAreaNameES().replaceAll("%", "/%").replaceAll("_", "/_"));
        }
        return tbAreaMapper.selectTbAreaList(tbArea);
    }

    @Override
    public List<AreaWithCountriesVo> selectTbAreaWithCountriesList(TbArea tbArea) {
        return tbAreaMapper.selectTbAreaWithCountriesList(tbArea);
    }
    
    /**
     * 查询区域列表（导出用）
     *
     * @param tbArea 区域
     * @return 区域
     */
    @Override
    public List<TbArea> selectTbAreaListExport(AreaParam tbArea)
    {
        return tbAreaMapper.selectTbAreaListExport(tbArea);
    }

    /**
     * 新增区域
     * 
     * @param tbArea 区域
     * @return 结果
     */
    @Override
    @Transactional
    @CacheEvict(value = AREA_KEY, allEntries = true)
    public int insertTbArea(TbArea tbArea)
    {
        // 校验区域名称
        int count = tbAreaMapper.selectTbAreaByAreaName(tbArea.getAreaName(), null);
        if (count > 0) {
           throw new ServiceException(MessageUtils.message("area.name.repeat"));
        }
        // 判断是否有上级区域
        if (tbArea.getParentArea() == null) {
            tbArea.setParentArea(0L);
        }
        // 区域ID采番
        tbArea.setAreaId(commonService.getCommonID("seq_area_agent"));
        // 数据权限字符
        tbArea.setPermissionChar(handlePermissionChar(tbArea));
        tbArea.setCreateBy(SecurityUtils.getUserId().toString());
        tbArea.setCreateTime(DateUtils.getNowDate());
        tbArea.setUpdateTime(DateUtils.getNowDate());
        int result = tbAreaMapper.insertTbArea(tbArea);
        // 插入国家区域关联表
        if (StringUtils.isNotEmpty(tbArea.getCountryIds())) {
            String[] countryIds = tbArea.getCountryIds().split(",");
            for (String countryId : countryIds) {
                tbCountryAreaMapper.bindCountryArea(tbArea.getAreaId(), Long.parseLong(countryId));
            }
        }
        return result;
    }

    /**
     * 拼接数据权限字符
     *
     * @param tbArea 区域
     * @return 数据权限字符
     */
    private String handlePermissionChar(TbArea tbArea) {
        String permissionChar;
        if (tbArea.getParentArea() == 0L) {
            permissionChar = "," + tbArea.getAreaId() + ",0,";
        } else {
            TbArea parentArea = tbAreaMapper.selectTbAreaByAreaId(tbArea.getParentArea());
            permissionChar = "," + tbArea.getAreaId() + parentArea.getPermissionChar();
        }
        return permissionChar;
    }

    /**
     * 修改区域
     * 
     * @param tbArea 区域
     * @return 结果
     */
    @Override
    @Transactional
    @CacheEvict(value = AREA_KEY, allEntries = true)
    public int updateTbArea(TbArea tbArea)
    {
        // 校验区域名称
        int count = tbAreaMapper.selectTbAreaByAreaName(tbArea.getAreaName(), tbArea.getAreaId());
        if (count > 0) {
            throw new ServiceException(MessageUtils.message("area.name.repeat"));
        }
        // 判断是否有上级区域
        if (tbArea.getParentArea() == null) {
            tbArea.setParentArea(0L);
        }
        // 数据权限字符
        tbArea.setPermissionChar(handlePermissionChar(tbArea));
        tbArea.setUpdateBy(SecurityUtils.getUserId().toString());
        tbArea.setUpdateTime(DateUtils.getNowDate());
        int result = tbAreaMapper.updateTbArea(tbArea);
        // 修改国家区域关系表
        tbCountryAreaMapper.deleteTbCountryAreaByAreaId(tbArea.getAreaId());
        if (StringUtils.isNotEmpty(tbArea.getCountryIds())) {
            String[] countryIds = tbArea.getCountryIds().split(",");
            for (String countryId : countryIds) {
                tbCountryAreaMapper.bindCountryArea(tbArea.getAreaId(), Long.parseLong(countryId));
            }
        }
        return result;
    }

    /**
     * 批量删除区域
     * 
     * @param areaIds 需要删除的区域主键
     * @return 结果
     */
    @Override
    @Transactional
    @CacheEvict(value = AREA_KEY, allEntries = true)
    public int deleteTbAreaByAreaIds(Long[] areaIds)
    {
        // 校验 当前区域若存在下级区域数据则无法删除
        int count = tbAreaMapper.selectTbAreaByParentAreas(areaIds);
        if (count > 0) {
            throw new ServiceException(MessageUtils.message("area.has.child"));
        }
        // 校验是否有客户使用
        int num = tcAccountMapper.selectCountByAreaIds(areaIds);
        if (num > 0) {
            throw new ServiceException(MessageUtils.message("account.cannot.delete"));
        }
        // 校验是否有代理商使用
        int agentCount = tbAgentMapper.selectCountByAreaIds(areaIds);
        if (agentCount > 0) {
            throw new ServiceException(MessageUtils.message("agent.cannot.delete"));
        }
        // 同时批量删除国家区域关系表信息
        tbCountryAreaMapper.deleteTbCountryAreaByAreaIds(areaIds);
        return tbAreaMapper.deleteTbAreaByAreaIds(areaIds);
    }

    /**
     * 删除区域信息
     * 
     * @param areaId 区域主键
     * @return 结果
     */
    @Override
    @CacheEvict(value = AREA_KEY, allEntries = true)
    public int deleteTbAreaByAreaId(Long areaId)
    {
        // 同时删除国家区域关系表信息
        tbCountryAreaMapper.deleteTbCountryAreaByAreaId(areaId);
        return tbAreaMapper.deleteTbAreaByAreaId(areaId);
    }

    /**
     * 获取所有区域列表
     * @return 结果
     */
    @Override
    public List<TbArea> getAllAreaList() {
        return tbAreaMapper.getAllAreaList();
    }

    /**
     * 根据区域等级获取区域列表
     * @return 结果
     */
    @Override
    public List<TbArea> getAreaListByLevel(String level) {
        return tbAreaMapper.getAreaListByLevel(level);
    }

    /**
     * 获取所有区域列表
     * @return 结果
     */
    @Override
    public List<Tree<String>> getAllAreaTree() {
        List<TbArea> areas = tbAreaMapper.getAllAreaList();
        
        // 如果不存在全球跟节点，手动添加
        boolean existRoot = areas.stream().anyMatch(area -> area.getAreaId() == 0L);
        if (!existRoot) {
            // 添加根节点：手动添加id：0；parent_id：-1作为全球（根节点）
            TbArea root = new TbArea();
            root.setAreaId(0L);
            root.setAreaName("全球");
            root.setParentArea(-1L);
            areas.add(root);
        }
        
        //TreeNodeConfig config = new TreeNodeConfig();
        // 最大递归深度 config.setDeep(5); 不配置默认无限递归

        return TreeUtil.build(areas, "-1", (node, tree) -> {
            tree.setId(node.getAreaId().toString());
            tree.setName(node.getAreaName());
            //tree.setParentId(node.getParentArea().toString());
            tree.setParentId(node.getParentArea() == null ? "-1" : node.getParentArea().toString());
            
            // 或者前端使用normalizer自定义属性名，默认使用label
            tree.put("node", node);
            tree.put("label", node.getAreaName());
            tree.put("labelRU", node.getAreaNameRU());
            tree.put("labelES", node.getAreaNameES());
            tree.put("labelUS", node.getAreaNameUS());
        });
    }

    @Override
    public List<TbArea> getAreaListByCountryId(Long countryId) {
        return tbAreaMapper.getAreaListByCountryId(countryId);
    }

    @Override
    public List<TbArea> selectAreaListByIds(List<Long> areaIds) {
        return tbAreaMapper.selectAreaListByIds(areaIds);
    }

    /**
     * 查询指定区域的所有子区域
     * TODO 这里缓存的 key 会不会有点多，可以优化
     * 
     * @param areaId 区域ID
     * @param justSubs 是否只查询子区域（不包括孙子即后代）
     * @param includeSelf 是否包含自己
     */
    @Override
    @Cacheable(value = AREA_KEY, key = "#areaId + '_' + #justSubs + '_' + #includeSelf", unless = "#result == null")
    public List<TbArea> selectChildAreaList(Long areaId, boolean justSubs, boolean includeSelf) {
        TbArea tbArea = tbAreaMapper.selectTbAreaByAreaId(areaId);
        if (tbArea == null) {
            throw new ServiceException(MessageUtils.message("area.name.invalid"));
        }
        List<TbArea> tbAreas;
        if (justSubs) {
            // 这里不用考虑上级区域是 null 的问题
            AreaParam areaParam = new AreaParam();
            areaParam.setParentArea(Collections.singletonList(areaId));
            tbAreas = tbAreaMapper.selectTbAreaList(areaParam);
            // 结果默认不包含自己
            if (includeSelf) {
                tbAreas.add(tbArea);
            }
        } else {
            // 反转字符串
            String permission = new StringBuilder(tbArea.getPermissionChar()).reverse().toString();
            //permission += justSubs ? "_," : "%";

            tbAreas = tbAreaMapper.selectChildAreaList(permission);
            // 结果默认包含自己
            if (!includeSelf) {
                tbAreas.removeIf(area -> area.getAreaId().equals(areaId));
            }
        }
       
        return tbAreas;
    }

    @Override
    public List<Long> selectChildAreaIdsByAreaId(Long areaId) {
        return tbAreaMapper.selectChildAreaIdsByAreaId(areaId);
    }
}
